
module convect_deep
!---------------------------------------------------------------------------------
! Purpose:
!
! CAM interface to several deep convection interfaces. Currently includes:
!    Zhang-McFarlane (default)
!    Kerry Emanuel 
!
!
! Author: D.B. Coleman, Sep 2004
!
!---------------------------------------------------------------------------------
   use shr_kind_mod, only: r8=>shr_kind_r8
   use ppgrid,       only: pver, pcols, pverp, begchunk, endchunk
   use cam_logfile,  only: iulog

   implicit none

   save
   private                         ! Make default type private to the module

! Public methods

   public ::&
      convect_deep_register,           &! register fields in physics buffer
      convect_deep_init,               &! initialize donner_deep module
      convect_deep_tend,               &! return tendencies
      convect_deep_tend_2,             &! return tendencies
      deep_scheme_does_scav_trans             ! = .t. if scheme does scavenging and conv. transport
   
! Private module data
   character(len=16) :: deep_scheme    ! default set in phys_control.F90, use namelist to change
! Physics buffer indices 
   integer     ::  icwmrdp_idx      = 0 
   integer     ::  rprddp_idx       = 0 
   integer     ::  nevapr_dpcu_idx  = 0 
   integer     ::  cldtop_idx       = 0 
   integer     ::  cldbot_idx       = 0 
   integer     ::  cld_idx          = 0 
   integer     ::  fracis_idx       = 0 

   integer     ::  pblh_idx        = 0 
   integer     ::  tpert_idx       = 0 
   integer     ::  prec_dp_idx     = 0
   integer     ::  snow_dp_idx     = 0

!=========================================================================================
  contains 

!=========================================================================================
function deep_scheme_does_scav_trans()
!
! Function called by tphysbc to determine if it needs to do scavenging and convective transport
! or if those have been done by the deep convection scheme. Each scheme could have its own
! identical query function for a less-knowledgable interface but for now, we know that KE 
! does scavenging & transport, and ZM doesn't
!

  logical deep_scheme_does_scav_trans

  deep_scheme_does_scav_trans = .false.

  if ( deep_scheme .eq. 'KE' ) deep_scheme_does_scav_trans = .true.

  return

end function deep_scheme_does_scav_trans

!=========================================================================================
subroutine convect_deep_register

!----------------------------------------
! Purpose: register fields with the physics buffer
!----------------------------------------

  
  use physics_buffer, only : pbuf_add_field, dtype_r8
  use zm_conv_intr, only: zm_conv_register
  use phys_control, only: phys_getopts
  use physconst,       only: numpft

  implicit none

  integer idx

  ! get deep_scheme setting from phys_control
  call phys_getopts(deep_scheme_out = deep_scheme)

  select case ( deep_scheme )
  case('ZM') !    Zhang-McFarlane (default)
     call zm_conv_register
  end select

  call pbuf_add_field('ICWMRDP_ENS',    'physpkg',dtype_r8,(/pcols,pver,numpft/),icwmrdp_idx)
  call pbuf_add_field('RPRDDP_ENS',     'physpkg',dtype_r8,(/pcols,pver,numpft/),rprddp_idx)
  call pbuf_add_field('NEVAPR_DPCU_ENS',     'physpkg',dtype_r8,(/pcols,pver,numpft/),nevapr_dpcu_idx)
  call pbuf_add_field('PREC_DP_ENS',   'physpkg',dtype_r8,(/pcols,numpft/),     prec_dp_idx)
  call pbuf_add_field('SNOW_DP_ENS',   'physpkg',dtype_r8,(/pcols,numpft/),     snow_dp_idx)

end subroutine convect_deep_register

!=========================================================================================



subroutine convect_deep_init(pref_edge)

!----------------------------------------
! Purpose:  declare output fields, initialize variables needed by convection
!----------------------------------------

  use pmgrid,        only: plevp
  use spmd_utils,    only: masterproc
  use zm_conv_intr,  only: zm_conv_init
  use abortutils,    only: endrun
  
  use physics_buffer,        only: physics_buffer_desc, pbuf_get_index

  implicit none

  real(r8),intent(in) :: pref_edge(plevp)        ! reference pressures at interfaces
  integer k

  select case ( deep_scheme )
  case('off') !     ==> no deep convection
     if (masterproc) write(iulog,*)'convect_deep: no deep convection selected'
  case('ZM') !    1 ==> Zhang-McFarlane (default)
     if (masterproc) write(iulog,*)'convect_deep initializing Zhang-McFarlane convection'
     call zm_conv_init(pref_edge)
  case default
     if (masterproc) write(iulog,*)'WARNING: convect_deep: no deep convection scheme. May fail.'
  end select

  cldtop_idx = pbuf_get_index('CLDTOP_ENS')
  cldbot_idx = pbuf_get_index('CLDBOT_ENS')
  cld_idx    = pbuf_get_index('CLD')
  fracis_idx = pbuf_get_index('FRACIS')

  pblh_idx   = pbuf_get_index('pblh_ens')
  tpert_idx  = pbuf_get_index('tpert_ens')

end subroutine convect_deep_init
!=========================================================================================
!subroutine convect_deep_tend(state, ptend, tdt, pbuf)

subroutine convect_deep_tend( &
     mcon    ,cme     ,          &
     dlf     ,pflx    ,zdu      , &
     rliq    , &
     ztodt   , &
     state   ,ptend   ,landfrac ,pbuf, output, lnd_i)

   use physics_types, only: physics_state, physics_ptend, physics_tend, physics_ptend_init
   
   use constituents, only: pcnst
   use zm_conv_intr, only: zm_conv_tend
#if ( defined WACCM_PHYS )
   use gw_drag,         only: idx_zmdt
   use cam_history,     only: outfld
   use physconst,       only: cpair
#endif
   use physics_buffer,          only: physics_buffer_desc, pbuf_get_field

! Arguments
   type(physics_state), intent(in ) :: state          ! Physics state variables
   type(physics_ptend), intent(out) :: ptend          ! indivdual parameterization tendencies
   

   type(physics_buffer_desc), pointer :: pbuf(:)
   real(r8), intent(in) :: ztodt                       ! 2 delta t (model time increment)
   real(r8), intent(in) :: landfrac(pcols)                ! Land fraction
   logical, intent(in) :: output    
   integer, intent(in) :: lnd_i    

   real(r8), intent(out) :: mcon(pcols,pverp)  ! Convective mass flux--m sub c
   real(r8), intent(out) :: dlf(pcols,pver)    ! scattrd version of the detraining cld h2o tend
   real(r8), intent(out) :: pflx(pcols,pverp)  ! scattered precip flux at each level
   real(r8), intent(out) :: cme(pcols,pver)    ! cmf condensation - evaporation
   real(r8), intent(out) :: zdu(pcols,pver)    ! detraining mass flux

   real(r8), intent(out) :: rliq(pcols) ! reserved liquid (not yet in cldliq) for energy integrals

   real(r8), pointer :: prec_ens(:,:)   ! total precipitation
   real(r8), pointer :: snow_ens(:,:)   ! snow from ZM convection 

   real(r8), pointer, dimension(:,:) :: jctop_ens
   real(r8), pointer, dimension(:,:) :: jcbot_ens
   real(r8), pointer, dimension(:,:,:) :: cld        
   real(r8), pointer, dimension(:,:,:) :: ql_ens           ! wg grid slice of cloud liquid water.
   real(r8), pointer, dimension(:,:,:) :: rprd_ens         ! rain production rate
   real(r8), pointer, dimension(:,:,:) :: fracis  ! fraction of transported species that are insoluble

   real(r8), pointer, dimension(:,:,:) :: evapcdp_ens      ! Evaporation of deep convective precipitation

   real(r8), pointer :: pblh_ens(:,:)                 ! Planetary boundary layer height
   real(r8), pointer :: tpert_ens(:,:)                ! Thermal temperature excess 

  real(r8) zero(pcols, pver)

  integer i, k

#if ( defined WACCM_PHYS )
   real(r8), pointer, dimension(:,:) :: zmdt
   real(r8) :: ftem(pcols,pver)              ! Temporary workspace for outfld variables
#endif

   call pbuf_get_field(pbuf, cldtop_idx, jctop_ens )
   call pbuf_get_field(pbuf, cldbot_idx, jcbot_ens )

  select case ( deep_scheme )
  case('off') !    0 ==> no deep convection
    zero = 0     
    mcon = 0
    dlf = 0
    pflx = 0
    cme = 0
    zdu = 0
    prec_ens = 0
    snow_ens = 0
    rliq = 0

!
! Associate pointers with physics buffer fields
!

    call pbuf_get_field(pbuf, cld_idx,         cld,    start=(/1,1/),   kount=(/pcols,pver/) ) 
    call pbuf_get_field(pbuf, icwmrdp_idx,     ql_ens )
    call pbuf_get_field(pbuf, rprddp_idx,      rprd_ens )
    call pbuf_get_field(pbuf, fracis_idx,      fracis, start=(/1,1,1/), kount=(/pcols, pver, pcnst/) )
    call pbuf_get_field(pbuf, nevapr_dpcu_idx, evapcdp_ens )
    call pbuf_get_field(pbuf, prec_dp_idx,     prec_ens )
    call pbuf_get_field(pbuf, snow_dp_idx,     snow_ens )


    jctop_ens(:,lnd_i) = pver
    jcbot_ens(:,lnd_i) = 1._r8
    cld = 0
    ql_ens(:,:,lnd_i) = 0
    rprd_ens(:,:,lnd_i) = 0
    fracis = 0
    evapcdp_ens(:,:,lnd_i) = 0



   call physics_ptend_init(ptend)
   ptend%name = "convect_deep"
  case('ZM') !    1 ==> Zhang-McFarlane (default)
     call pbuf_get_field(pbuf, pblh_idx,  pblh_ens)
     call pbuf_get_field(pbuf, tpert_idx, tpert_ens)

     call zm_conv_tend( pblh_ens(:,lnd_i)    ,mcon    ,cme     , &
          tpert_ens(:,lnd_i)   ,dlf     ,pflx    ,zdu      , &
          rliq    , &
          ztodt   , &
          jctop_ens(:,lnd_i), jcbot_ens(:,lnd_i) , &
          state   ,ptend   ,landfrac, pbuf, output, lnd_i)

  end select

#if ( defined WACCM_PHYS )
   call pbuf_get_field(pbuf, idx_zmdt, zmdt)
   ftem(:state%ncol,:pver) = ptend%s(:state%ncol,:pver)/cpair
   zmdt(:state%ncol,:pver) = ftem(:state%ncol,:pver)
   if(output) then
      call outfld('ZMDT    ',ftem           ,pcols   ,state%lchnk   )
      call outfld('ZMDQ    ',ptend%q(:,:,1) ,pcols   ,state%lchnk   )
   end if
#endif


end subroutine convect_deep_tend
!=========================================================================================


subroutine convect_deep_tend_2( state,  ptend,  ztodt, pbuf)

   use physics_types, only: physics_state, physics_ptend
   
   use physics_buffer,  only: physics_buffer_desc
   use constituents, only: pcnst
   use zm_conv_intr, only: zm_conv_tend_2

! Arguments
   type(physics_state), intent(in ) :: state          ! Physics state variables
   type(physics_ptend), intent(out) :: ptend          ! indivdual parameterization tendencies
   
   type(physics_buffer_desc), pointer :: pbuf(:)

   real(r8), intent(in) :: ztodt                          ! 2 delta t (model time increment)

   if ( deep_scheme .eq. 'ZM' ) then  !    1 ==> Zhang-McFarlane (default)
      call zm_conv_tend_2( state,   ptend,  ztodt,  pbuf) 
   end if


end subroutine convect_deep_tend_2


end module convect_deep
